@@ -15,13 +15,7 @@ describe processor do
   subject {
     s = Puppet::Transaction::Report.new("foo").extend(processor)
     s.configuration_version = 123456789
-
-    # For backwards compatibility with versions of Puppet that don't
-    # have an accessor method for the report_format variable
-    if !s.respond_to?(:report_format)
-      s.stubs(:report_format).returns(s.instance_variable_get(:@report_format))
-    end
-
+    s.environment = "foo"
     s
   }
 
@@ -36,13 +30,13 @@ describe processor do
 
       payload = {
         :command => Puppet::Util::Puppetdb::CommandNames::CommandStoreReport,
-        :version => 3,
+        :version => 4,
         :payload => subject.send(:report_to_hash),
       }.to_json
 
       Puppet::Network::HttpPool.expects(:http_instance).returns(http)
       http.expects(:post).with {|path, body, headers|
-        expect(path).to include(Puppet::Util::Puppetdb::Command::Url)
+        expect(path).to include(Puppet::Util::Puppetdb::Command::CommandsUrl)
         expect(body).to eq(payload)
       }.returns(httpok)
 
@@ -70,15 +64,8 @@ describe processor do
       subject.add_resource_status(status)
     end
 
-    it "should error on reports with no metrics" do
-      # at this point subject has no metrics
-      expect { subject.send(:report_to_hash) }.to raise_error(Puppet::Error)
-    end
-
     it "should include the transaction uuid or nil" do
       # Prevents subject.send(:report_to_hash) from exploding
-      subject.stubs(:run_duration).returns(-1)
-
       if subject.report_format >= 4
         subject.transaction_uuid = 'abc123'
         result = subject.send(:report_to_hash)
@@ -118,9 +105,8 @@ describe processor do
           # client doesn't include one
           result.has_key?("report").should be_false
           result["certname"].should == subject.host
-          # TODO: change these two to use accessors as soon as we get up to puppet 3.0
-          result["puppet-version"].should == subject.instance_variable_get(:@puppet_version)
-          result["report-format"].should == subject.instance_variable_get(:@report_format)
+          result["puppet-version"].should == subject.puppet_version
+          result["report-format"].should == subject.report_format
           result["configuration-version"].should == subject.configuration_version.to_s
           result["resource-events"].should == []
         end
@@ -177,19 +163,9 @@ describe processor do
         end
 
         context "with no events" do
-          it "should include a fabricated event" do
+          it "should include have no events" do
             result = subject.send(:report_to_hash)
-            result["resource-events"].length.should == 1
-            event = result["resource-events"][0]
-            event["status"].should == "failure"
-            event["resource-type"].should == "Foo"
-            event["resource-title"].should == "foo"
-            event["property"].should be_nil
-            event["new-value"].should be_nil
-            event["old-value"].should be_nil
-            event["message"].should be_nil
-            event["file"].should == "foo"
-            event["line"].should == 1
+            result["resource-events"].length.should == 0
           end
         end
 
