@@ -204,14 +204,33 @@
           (catch Exception e#
             (log/error e# "Error streaming response")))))))
 
-(defn stream-json-results
-  "Encapsulates streaming results from `f` to a piped-input-stream.
-   `f` is a function of one argument, which is another function. The
-    the function it accepts will accept one argument that is an individual
-    result (i.e. a single row) that should be streamed."
+(defn stream-json-response
+  "Converts streaming results from function `f` into a streaming Ring response.
+
+  This works in tandem with query/query-stream-results. So usually `f` is the
+  result of a call to this function, normally to achieve streaming from a DB
+  cursor.
+
+  It wraps the execution of `f` and subsequent results in a Ring
+  piped-input-stream thread allowing the cursor stream and JSON encoding to
+  continue while the web server starts serving the content immediately (using
+  chunked encoding usually).
+
+  `f` is a function of one argument, which is another function. The
+  the function `f` will accept one argument that is a LazySeq
+  result set. This result set will be piped through a JSON stream.
+
+  Returns a Ring response map with the :body containing a Buffer. Processing
+  and conversion into the JSON stream will continue in another thread and update
+  the Buffer as results are returned."
   [f]
+  {:post [(rr/response? %)]}
+  ;; json-response here creates a Ring response putting the Buffer into
+  ;; the :body so that ring can stream from it.
   (json-response*
    (streamed-response buffer
+     ;; We pass the stream function back to the stream-query-result so that
+     ;; it gets executed inside the cursor function.
      (f #(stream-json % buffer)))))
 
 (defn parse-boolean-query-param
