@@ -5,19 +5,22 @@
             [com.puppetlabs.utils.metrics :refer [multitime!]]
             [com.puppetlabs.http :as pl-http]
             [ring.util.response :as rr]
+            [ring.util.request :as request]
             [clojure.string :as s]
             [clojure.tools.logging :as log]
             [com.puppetlabs.puppetdb.query.paging :as paging]
-            [clojure.set :as set])
+            [clojure.set :as set]
+            [pantomime.media :as media]
+            [com.puppetlabs.jdbc :refer [with-transacted-connection]])
   (:use [metrics.timers :only (timer time!)]
         [metrics.meters :only (meter mark!)]
         [clojure.walk :only (keywordize-keys)]))
 
 (defn wrap-with-debug-logging
   "Ring middleware that logs incoming HTTP request URIs (at DEBUG level) as
-  requests come in.  To enable, add this line to your log4j.properties:
+  requests come in.  To enable, add this line to your logback.xml:
 
-  `log4j.logger.com.puppetlabs.middleware=DEBUG`"
+  `<logger name=\"com.puppetlabs.middleware\" level=\"debug\"/>`"
   [app]
   (fn [req]
     (log/debug (str "Processing HTTP request to URI: '" (:uri req) "'"))
@@ -69,6 +72,13 @@
     (let [new-req (assoc req :globals globals)]
       (app new-req))))
 
+(defn wrap-read-db-tx
+  "Ring middleware that creates a database transaction on the read-db"
+  [handler]
+  (fn [{:keys [globals] :as req}]
+    (with-transacted-connection (:scf-read-db globals)
+      (handler req))))
+
 (defn wrap-with-paging-options
   "Ring middleware that will add to each request a :paging-options attribute:
   a map optionally containing :limit, :offset, and :order-by keys used to
@@ -105,6 +115,20 @@
       (rr/status (rr/response (str "must accept " content-type))
                  pl-http/status-not-acceptable))))
 
+(defn verify-content-type
+  "Verification for the specified list of content-types."
+  [app content-types]
+  {:pre [(coll? content-types)
+         (every? string? content-types)]}
+  (fn [{:keys [headers] :as req}]
+    (let [content-type (headers "content-type")
+          mediatype (if (nil? content-type) nil
+                        (str (media/base-type content-type)))]
+      (if (or (nil? mediatype) (some #{mediatype} content-types))
+        (app req)
+        (rr/status (rr/response (str "content type " mediatype " not supported"))
+                   pl-http/status-unsupported-type)))))
+
 (defn validate-query-params
   "Ring middleware that verifies that the query params in the request
   are legal based on the map `param-specs`, which contains a list of
@@ -116,19 +140,19 @@
          (= #{} (kitchensink/keyset (dissoc param-specs :required :optional)))
          (every? string? (:required param-specs))
          (every? string? (:optional param-specs))]}
-    (fn [{:keys [params] :as req}]
-      (kitchensink/cond-let [p]
-        (kitchensink/excludes-some params (:required param-specs))
-        (pl-http/error-response (str "Missing required query parameter '" p "'"))
+  (fn [{:keys [params] :as req}]
+    (kitchensink/cond-let [p]
+      (kitchensink/excludes-some params (:required param-specs))
+      (pl-http/error-response (str "Missing required query parameter '" p "'"))
 
-        (let [diff (set/difference (kitchensink/keyset params)
-                      (set (:required param-specs))
-                      (set (:optional param-specs)))]
-          (when (seq diff) diff))
-        (pl-http/error-response (str "Unsupported query parameter '" (first p) "'"))
+      (let [diff (set/difference (kitchensink/keyset params)
+                    (set (:required param-specs))
+                    (set (:optional param-specs)))]
+        (seq diff))
+      (pl-http/error-response (str "Unsupported query parameter '" (first p) "'"))
 
-        :else
-        (app req))))
+      :else
+      (app req))))
 
 (defn validate-no-query-params
   "Ring middleware that verifies that there are no query params on the request.
@@ -147,14 +171,14 @@
    (verify-accepts-content-type app "application/json")))
 
 (defn verify-checksum
-  "Ring middleware which will verify that the content of the `payload` param
+  "Ring middleware which will verify that the content of the body-string
   has the checksum specified in the `checksum` parameter. If no checksum is
   provided, this check will be skipped. If the checksum doesn't match, a 400
   Bad Request error is returned."
   [app]
-  (fn [{:keys [params] :as req}]
+  (fn [{:keys [params body-string] :as req}]
     (let [expected-checksum (params "checksum")
-          payload           (params "payload")]
+          payload           body-string]
       (if (and expected-checksum
                (not= expected-checksum (kitchensink/utf8-string->sha1 payload)))
         (pl-http/error-response "checksums don't match")
@@ -229,3 +253,27 @@
   [app storage normalize-uri]
   `(let [prefix# ~(str *ns*)]
      (wrap-with-metrics* ~app prefix# ~storage ~normalize-uri)))
+
+(defn payload-to-body-string
+  "Middleware to move the payload from the body or the payload param into the
+  request property `:body-string`.
+
+  Usually the body is an InputStream, it is first converted to a string."
+  [app]
+  (fn [{:keys [body params headers] :as req}]
+    (let [content-type (headers "content-type")
+          mediatype (if (nil? content-type) nil
+                        (str (media/base-type content-type)))]
+      (case mediatype
+        "application/x-www-form-urlencoded"
+          (if-let [payload (params "payload")]
+            (app (assoc req :body-string payload))
+            (pl-http/error-response (str "Missing required parameter 'payload'")))
+        "application/json"
+          (let [body-string (request/body-string req)]
+            (if (nil? body-string)
+              (pl-http/error-response (str "Empty body for application/json submission"))
+              (app (assoc req :body-string body-string))))
+        (if-let [payload (params "payload")]
+          (app (assoc req :body-string payload))
+          (pl-http/error-response (str "Missing required parameter 'payload'")))))))
