@@ -41,16 +41,18 @@
   (:require [clojure.tools.logging :as log]
             [com.puppetlabs.puppetdb.catalogs :as cat]
             [com.puppetlabs.puppetdb.catalog.utils :as catutils]
-            [com.puppetlabs.utils.logging :as logutils]
+            [puppetlabs.trapperkeeper.logging :as logutils]
             [com.puppetlabs.puppetdb.command :as command]
             [com.puppetlabs.http :as pl-http]
             [com.puppetlabs.cheshire :as json]
             [clj-http.client :as client]
             [clj-http.util :as util]
-            [fs.core :as fs])
-  (:use [puppetlabs.kitchensink.core :only (cli! inis-to-map utf8-string->sha1)]
-        [com.puppetlabs.puppetdb.scf.migrate :only [migrate!]]
-        [com.puppetlabs.puppetdb.command.constants :only [command-names]]))
+            [fs.core :as fs]
+            [com.puppetlabs.puppetdb.utils :as utils]
+            [puppetlabs.kitchensink.core :as kitchensink]
+            [clj-time.core :as time]
+            [com.puppetlabs.puppetdb.command.constants :refer [command-names]]
+            [slingshot.slingshot :refer [try+]]))
 
 (def cli-description "Development-only benchmarking tool")
 
@@ -76,7 +78,7 @@
   command-processing endpoint located at `puppetdb-host`:`puppetdb-port`."
   [puppetdb-host puppetdb-port catalog]
   (let [result (command/submit-command-via-http! puppetdb-host puppetdb-port
-                 (command-names :replace-catalog) 1 (json/generate-string catalog))]
+                 (command-names :replace-catalog) 4 (json/generate-string catalog))]
     (when-not (= pl-http/status-ok (:status result))
       (log/error result))))
 
@@ -86,19 +88,41 @@
   [puppetdb-host puppetdb-port report]
   (let [result (command/submit-command-via-http!
                  puppetdb-host puppetdb-port
-                 (command-names :store-report) 1 report)]
+                 (command-names :store-report) 3 report)]
     (when-not (= pl-http/status-ok (:status result))
       (log/error result))))
 
+(def mutate-fns
+  "Functions that randomly change a wire-catalog format"
+  [catutils/add-random-resource-to-wire-catalog
+   catutils/mod-resource-in-wire-catalog
+   catutils/add-random-edge-to-wire-catalog
+   catutils/swap-edge-targets-in-wire-catalog])
+
+(defn rand-catalog-mutate-fn
+  "Grabs one of the mutate-fns randomly and returns it"
+  []
+  (rand-nth mutate-fns))
+
 (defn maybe-tweak-catalog
   "Slightly tweak the given catalog, returning a new catalog, `rand-percentage`
   percent of the time."
   [rand-percentage catalog]
   (if (< (rand 100) rand-percentage)
-    (catutils/add-random-resource-to-wire-catalog catalog)
+    ((rand-catalog-mutate-fn) catalog)
     catalog))
 
-(defn update-host
+(defn update-report-run-fields
+  "configuration-version, start-time and end-time should always change
+   on subsequent report submittions, this changes those fields to avoid
+   computing the same hash again (causing constraint errors in the DB)"
+  [report]
+  (assoc report
+    "configuration-version" (kitchensink/uuid)
+    "start-time" (time/now)
+    "end-time" (time/now)))
+
+(defn timed-update-host
   "Send a new _clock tick_ to a host
 
   On each tick, a host:
@@ -113,7 +137,8 @@
   * Submit the resulting catalog"
   [{:keys [host lastrun catalog report puppetdb-host puppetdb-port run-interval rand-percentage] :as state} clock]
   (if (> (- clock lastrun) run-interval)
-    (let [catalog (if catalog (maybe-tweak-catalog rand-percentage catalog))]
+    (let [catalog (if catalog (maybe-tweak-catalog rand-percentage catalog))
+          report (and report (update-report-run-fields report))]
       ;; Submit the catalog and reports in separate threads, so as to not
       ;; disturb the world-loop and otherwise distort the space-time continuum.
       (when catalog
@@ -133,6 +158,31 @@
       (assoc state :lastrun clock :catalog catalog))
     state))
 
+(defn update-host
+  "Submit a `catalog` for `hosts` (when present), possibly mutating it before
+   submission.  Also submit a report for the host (if present). This is
+   similar to timed-update-host, but always sends the update (doesn't run/skip
+   based on the clock)"
+  [{:keys [host lastrun catalog report puppetdb-host puppetdb-port run-interval rand-percentage] :as state}]
+  (let [catalog (and catalog (maybe-tweak-catalog rand-percentage catalog))
+        report (and report (update-report-run-fields report))]
+    (when catalog
+      (submit-catalog puppetdb-host puppetdb-port catalog))
+    (when report
+      (submit-report puppetdb-host puppetdb-port report))
+    (assoc state :catalog catalog)))
+
+(defn submit-n-messages
+  "Given a list of host maps, send `num-messages` to each host.  The function
+   is recursive to accumulate possible catalog mutations (i.e. changing a previously
+   mutated catalog as opposed to different mutations of the same catalog)."
+  [hosts num-msgs]
+  (printf "Sending %s messages for %s hosts, will exit upon completion" num-msgs hosts)
+  (loop [mutated-hosts hosts
+         msgs-to-send num-msgs]
+    (when-not (zero? msgs-to-send)
+      (recur (mapv update-host mutated-hosts) (dec msgs-to-send)))))
+
 (defn world-loop
   "Sends out new _clock tick_ messages to all agents.
 
@@ -145,7 +195,7 @@
 
       ;; Send out updated ticks to each agent
       (doseq [host hosts]
-        (send host update-host curr-time)
+        (send host timed-update-host curr-time)
         (when-let [error (agent-error host)]
           (log/error error (format "[%s] agent failed; restarting" host))
           ;; Restart it with exactly the same state. Hopefully that's okay!
@@ -158,10 +208,9 @@
   "Takes the given `catalog` and transforms it to appear related to
   `hostname`"
   [hostname catalog]
-  (-> catalog
-      (assoc-in ["data" "name"] hostname)
-      (assoc-in ["data" "resources"] (for [resource (get-in catalog ["data" "resources"])]
-                                       (update-in resource ["tags"] conj hostname)))))
+  (assoc catalog
+    "name" hostname
+    "resources" (map #(update-in % ["tags"] conj hostname) (get catalog "resources"))))
 
 (defn associate-report-with-host
   "Takes the given `report` and transforms it to appear related to
@@ -169,25 +218,44 @@
   [hostname report]
   (assoc report "certname" hostname))
 
-
 (def supported-cli-options
-  [["-c" "--config" "Path to config.ini or conf.d directory (required)"]
-   ["-C" "--catalogs" "Path to a directory containing sample JSON catalogs (files must end with .json)"]
-   ["-R" "--reports" "Path to a directory containing sample JSON reports (files must end with .json)"]
-   ["-i" "--runinterval" "What runinterval (in minutes) to use during simulation"]
-   ["-n" "--numhosts" "How many hosts to use during simulation"]
-   ["-rp" "--rand-perc" "What percentage of submitted catalogs are tweaked (int between 0 and 100)"]])
+  [["-c" "--config CONFIG" "Path to config.ini or conf.d directory (required)"]
+   ["-C" "--catalogs CATALOGS" "Path to a directory containing sample JSON catalogs (files must end with .json)"]
+   ["-R" "--reports REPORTS" "Path to a directory containing sample JSON reports (files must end with .json)"]
+   ["-i" "--runinterval RUNINTERVAL" "What runinterval (in minutes) to use during simulation"]
+   ["-n" "--numhosts NUMHOSTS" "How many hosts to use during simulation"]
+   ["-rp" "--rand-perc RANDPERC" "What percentage of submitted catalogs are tweaked (int between 0 and 100)"]
+   ["-N" "--nummsgs NUMMSGS" "Number of commands and/or reports to send for each host"]])
 
 (def required-cli-options
   [:config])
 
+(defn- validate-cli!
+  [args]
+  (try+
+    (kitchensink/cli! args supported-cli-options required-cli-options)
+    (catch map? m
+      (println (:message m))
+      (case (:type m)
+        :puppetlabs.kitchensink.core/cli-error (System/exit 1)
+        :puppetlabs.kitchensink.core/cli-help (System/exit 0)))))
+
+(defn validate-nummsgs [options action-on-error-fn]
+  (when (and (contains? options :runinterval)
+             (contains? options :nummsgs))
+    (utils/println-err "Error: -N/--nummsgs runs immediately and is not compatable with -i/--runinterval")
+    (action-on-error-fn)))
+
 (defn -main
   [& args]
-  (let [[options _]     (cli! args supported-cli-options required-cli-options)
+  (let [[options _]     (kitchensink/cli! args supported-cli-options required-cli-options)
         config          (-> (:config options)
-                            (inis-to-map)
+                            (kitchensink/inis-to-map)
+                            (get-in [:global :logging-config])
                             (logutils/configure-logging!))
 
+        _ (validate-nummsgs options #(System/exit 1))
+
         catalogs        (if (:catalogs options) (load-sample-data (:catalogs options)))
         reports         (if (:reports options) (load-sample-data (:reports options)))
 
@@ -196,22 +264,28 @@
         hostname        (get-in config [:jetty :host] "localhost")
         port            (get-in config [:jetty :port] 8080)
         rand-percentage (Integer/parseInt (:rand-perc options))
-        run-interval     (* 60 1000 (Integer/parseInt (:runinterval options)))
 
         ;; Create an agent for each host
-        hosts (mapv #(agent {:host    %
-                             :puppetdb-host hostname
-                             :puppetdb-port port
-                             :rand-percentage rand-percentage
-                             :run-interval run-interval
-                             :lastrun (- (System/currentTimeMillis) (rand-int run-interval))
-                             :catalog (if catalogs (associate-catalog-with-host % (rand-nth catalogs)))
-                             :report (if reports (associate-report-with-host % (rand-nth reports)))})
+        hosts (mapv (fn [host]
+                      {:host host
+                       :puppetdb-host hostname
+                       :puppetdb-port port
+                       :rand-percentage rand-percentage
+                       :catalog (when catalogs
+                                  (associate-catalog-with-host host (rand-nth catalogs)))
+                       :report (when reports
+                                 (associate-report-with-host host (rand-nth reports)))})
                     hostnames)]
 
     (when-not catalogs
       (log/info "No catalogs specified; skipping catalog submission"))
     (when-not reports
       (log/info "No reports specified; skipping report submission"))
-    ;; Loop forever
-    (world-loop hosts)))
+    (if-let [num-cmds (:nummsgs options)]
+      (submit-n-messages hosts (Long/valueOf num-cmds))
+      (world-loop (mapv (fn [host-map]
+                          (let [run-interval (* 60 1000 (Integer/parseInt (:runinterval options)))]
+                            (agent (assoc host-map
+                                     :run-interval run-interval
+                                     :lastrun (- (System/currentTimeMillis) (rand-int run-interval))))))
+                        hosts)))))
