@@ -6,6 +6,7 @@
             [com.puppetlabs.puppetdb.http.facts :as facts]
             [com.puppetlabs.puppetdb.http.resources :as resources]
             [com.puppetlabs.puppetdb.http.nodes :as nodes]
+            [com.puppetlabs.puppetdb.http.environments :as envs]
             [net.cgrand.moustache :as moustache]))
 
 (def version :v2)
@@ -13,7 +14,7 @@
 (def v2-app
   (moustache/app
    ["commands"]
-   {:any cmd/command}
+   {:any (cmd/command-app version)}
 
    ["facts" &]
    {:any (facts/facts-app version)}
@@ -24,6 +25,9 @@
    ["nodes" &]
    {:any (nodes/node-app version)}
 
+   ["environments" &]
+   {:any (envs/environments-app version)}
+
    ["resources" &]
    {:any (resources/resources-app version)}
 
