@@ -67,12 +67,11 @@
             [puppetlabs.kitchensink.core :as kitchensink]
             [com.puppetlabs.jdbc :as jdbc]
             [clj-time.coerce :refer [to-timestamp]]
-            [com.puppetlabs.puppetdb.http :refer [remove-all-environments]])
-  (:use [puppetlabs.kitchensink.core :only [parse-number keyset valset order-by-expr?]]
-        [com.puppetlabs.puppetdb.scf.storage-utils :only [db-serialize sql-as-numeric sql-array-query-string sql-regexp-match sql-regexp-array-match]]
-        [com.puppetlabs.jdbc :only [valid-jdbc-query? limited-query-to-vec query-to-vec paged-sql count-sql get-result-count]]
-        [com.puppetlabs.puppetdb.query.paging :only [requires-paging?]]
-        [clojure.core.match :only [match]]))
+            [puppetlabs.kitchensink.core :refer [parse-number keyset valset order-by-expr?]]
+            [com.puppetlabs.puppetdb.scf.storage-utils :refer [db-serialize sql-as-numeric sql-array-query-string sql-regexp-match sql-regexp-array-match]]
+            [com.puppetlabs.jdbc :refer [valid-jdbc-query? limited-query-to-vec query-to-vec paged-sql count-sql get-result-count]]
+            [com.puppetlabs.puppetdb.query.paging :refer [requires-paging?]]
+            [clojure.core.match :refer [match]]))
 
 (defn execute-paged-query*
   "Helper function to executed paged queries.  Builds up the paged sql string,
@@ -266,6 +265,21 @@
    "containing_class"       ["resource_events"]
    "name"                   ["environments" "environment"]})
 
+(def report-columns
+  "Return the queryable set of fields and corresponding table names where they reside"
+  {"hash"                  "reports"
+   "certname"              "reports"
+   "puppet_version"        "reports"
+   "report_format"         "reports"
+   "configuration_version" "reports"
+   "start_time"            "reports"
+   "end_time"              "reports"
+   "receive_time"          "reports"
+   "transaction_uuid"      "reports"
+   "environment"           "reports"
+   "status"                "reports"})
+
+
 (defn column-map->sql
   "Helper function that converts one of our column maps to a SQL string suitable
   for use in a SELECT"
@@ -305,6 +319,10 @@
   [_ _]
   (keyset event-columns))
 
+(defmethod queryable-fields :report
+  [_ _]
+  (keyset report-columns))
+
 (def subquery->type
   {"select-resources" :resource
    "select-facts"     :fact})
@@ -424,6 +442,34 @@
                        (format "WHERE %s"  where))]
     (apply vector (str projection where-clause) params)))
 
+(defn report-query->sql
+  "Compile a node query, returning a vector containing the SQL and parameters
+  for the query. All node columns are selected, and no order is applied."
+  [version ops query]
+  {:post [valid-jdbc-query? %]}
+  (let [sql (format "SELECT %s FROM (SELECT
+                     reports.hash,
+                     reports.certname,
+                     reports.puppet_version,
+                     reports.report_format,
+                     reports.configuration_version,
+                     reports.start_time,
+                     reports.end_time,
+                     reports.receive_time,
+                     reports.transaction_uuid,
+                     environments.name as environment,
+                     report_statuses.status as status
+                     FROM reports
+                       LEFT OUTER JOIN environments on reports.environment_id = environments.id
+                       LEFT OUTER JOIN report_statuses on reports.status_id = report_statuses.id) as reports"
+                    (column-map->sql report-columns))]
+    (if query
+      (let [{:keys [where params]} (compile-term ops query)
+            ;; This order by is for legacy v3 to continue to work the same way
+            sql (str sql (format " WHERE %s ORDER BY start_time DESC" where))]
+        (apply vector sql params))
+      (vector sql))))
+
 (defn compile-resource-equality
   "Compile an = operator for a resource query. `path` represents the field
   to query against, and `value` is the value. This mostly just defers to
@@ -492,7 +538,7 @@
           (compile-resource-regexp :v3 (get v3-renamed-resource-columns path path) value))
     (match [path]
            ["tag"]
-           {:where (sql-regexp-array-match "catalog_resources" "tags")
+           {:where (sql-regexp-array-match "catalog_resources" "catalog_resources" "tags")
             :params [value]}
 
            ;; node join.
@@ -788,6 +834,69 @@ args))))))
              :else (throw (IllegalArgumentException.
                            (format "'%s' is not a queryable object for version %s of the resource events API" path (last (name version)))))))))
 
+(defn compile-reports-equality
+  "Compile a report query into a structured map reflecting the terms
+   of the query. Currently only the `=` operator is supported"
+  [version]
+  (fn [& [path value :as term]]
+    {:post [(map? %)
+            (string? (:where %))]}
+    (let [num-args (count term)]
+      (when-not (= 2 num-args)
+        (throw (IllegalArgumentException.
+                (format "= requires exactly two arguments, but we found %d" num-args)))))
+    (match [path]
+           ["certname"]
+           {:where "reports.certname = ?"
+            :params [value] }
+
+           ["hash"]
+           {:where "reports.hash = ?"
+            :params [value]}
+
+           ["environment" :guard (v4? version)]
+           {:where "environments.name = ?"
+            :params [value]}
+
+           ["status" :guard (v4? version)]
+           {:where "report_statuses.status = ?"
+            :params [value]}
+
+           :else
+           (throw (IllegalArgumentException.
+                   (format "'%s' is not a valid query term for version %s of the reports API" path (last (name version))))))))
+
+(defn compile-event-count-equality
+  "Compile an = predicate for event-count query.  The `path` represents
+  the field to query against, and `value` is the value of the field."
+  [& [path value :as args]]
+  {:post [(map? %)
+          (string? (:where %))]}
+  (when-not (= (count args) 2)
+    (throw (IllegalArgumentException. (format "= requires exactly two arguments, but %d were supplied" (count args)))))
+  (let [db-field (jdbc/dashes->underscores path)]
+    (match [db-field]
+      [(field :guard #{"successes" "failures" "noops" "skips"})]
+      {:where (format "%s = ?" field)
+       :params [value]}
+
+      :else (throw (IllegalArgumentException. (str path " is not a queryable object for event counts"))))))
+
+(defn compile-event-count-inequality
+  "Compile an inequality for an event-counts query (> < >= <=).  The `path`
+  represents the field to query against, and the `value` is the value of the field."
+  [& [op path value :as args]]
+  {:post [(map? %)
+          (string? (:where %))]}
+  (when-not (= (count args) 3)
+    (throw (IllegalArgumentException. (format "%s requires exactly two arguments, but %d were supplied" op (dec (count args))))))
+  (match [path]
+    [(field :guard #{"successes" "failures" "noops" "skips"})]
+    {:where (format "%s %s ?" field op)
+     :params [value]}
+
+    :else (throw (IllegalArgumentException. (format "%s operator does not support object '%s' for event counts" op path)))))
+
 (declare fact-operators)
 
 (defn resource-operators
@@ -904,12 +1013,50 @@ args))))))
           (= op "select-resources") (partial resource-query->sql (resource-operators version))
           (= op "select-facts") (partial fact-query->sql (fact-operators version)))))))
 
+(defn report-ops
+  [version]
+  (case version
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+    (fn [op]
+      (let [op (string/lower-case op)]
+        (cond
+         (= op "=") (compile-reports-equality version)
+         (= op "and") (partial compile-and (report-ops version)))))))
+
+(defn event-count-ops
+  "Maps resource event count operators to the functions implementing them.
+  Returns nil if the operator is unknown."
+  [op]
+  (let [op (string/lower-case op)]
+    (cond
+      (= "=" op) compile-event-count-equality
+      (#{">" "<" ">=" "<="} op) (partial compile-event-count-inequality op))))
+
+(defn remove-environment
+  "dissocs the :environment key when the version is :v4"
+  [result-map version]
+  (if-not (= :v4 version)
+    (dissoc result-map :environment)
+    result-map))
+
+(defn remove-all-environments
+  "Removes environment from a seq of results"
+  [version rows]
+  (map #(remove-environment % version) rows))
+
 (defn streamed-query-result
   "Uses a cursored resultset (for streaming), removing environments when not
-   in version ;v4. Returns a function that accepts a single function. That function
-   with get the results of the query"
-  [db version sql params]
-  (fn [f]
-    (jdbc/with-transacted-connection db
-      (jdbc/with-query-results-cursor sql params rs
-        (f (remove-all-environments version rs))))))
+   in version :v4. Returns the results after running the function `f` with the
+   resultset.
+
+   That function will get the results of the query. Ordinarily some sort of
+   munging and finally serialization is performed within this function. See
+   http/stream-json-response for a function to provide JSON streaming for
+   example.
+
+   If all you want is an unstreamed Seq, pass the function `doall` as `f` to
+   convert the LazySeq to a Seq by full traversing it. This is useful for tests,
+   that cannot analyze results easily in a streamed way."
+  [version sql params f]
+  (jdbc/with-query-results-cursor sql params rs
+    (f (remove-all-environments version rs))))
