@@ -1,12 +1,10 @@
 ;; ## Fact query generation
 
 (ns com.puppetlabs.puppetdb.query.facts
-  (:refer-clojure :exclude [case compile conj! distinct disj! drop sort take])
-  (:require [clojure.string :as string]
-            [com.puppetlabs.jdbc :as sql]
-            [com.puppetlabs.puppetdb.http :refer [remove-environment remove-all-environments]])
-  (:use [com.puppetlabs.puppetdb.query :only [fact-query->sql fact-operators execute-query]]
-        [com.puppetlabs.puppetdb.query.paging :only [validate-order-by!]]))
+  (:require [com.puppetlabs.jdbc :as jdbc]
+            [com.puppetlabs.puppetdb.query :as query]
+            [com.puppetlabs.puppetdb.query.paging :as paging]
+            [com.puppetlabs.puppetdb.query-eng :as qe]))
 
 (defn flat-facts-by-node
   "Similar to `facts-for-node`, but returns facts in the form:
@@ -15,7 +13,7 @@
      ...
      {:certname <node> :name <fact> :value <value>}]"
   [node]
-  (sql/query-to-vec
+  (jdbc/query-to-vec
     ["SELECT certname, name, value FROM certname_facts WHERE certname = ?"
      node]))
 
@@ -28,8 +26,8 @@
     {:post [(map? %)
             (coll? (:result %))
             (every? string? (:result %))]}
-    (validate-order-by! [:name] paging-options)
-    (let [facts (execute-query
+    (paging/validate-order-by! [:name] paging-options)
+    (let [facts (query/execute-query
                   ["SELECT DISTINCT name FROM certname_facts ORDER BY name"]
                   paging-options)]
       (update-in facts [:result] #(map :name %)))))
@@ -39,7 +37,7 @@
    needed for that query as the rest."
   [operators query paging-options]
   (if query
-    (let [[subselect & params] (fact-query->sql operators query)
+    (let [[subselect & params] (query/fact-query->sql operators query)
           sql (format "SELECT facts.certname, facts.environment, facts.name, facts.value FROM (%s) facts" subselect)]
       (apply vector sql params))
     ["SELECT certname, name, value FROM certname_facts"]))
@@ -51,8 +49,14 @@
    :post [(map? %)
           (string? (first (:results-query %)))
           (every? (complement coll?) (rest (:results-query %)))]}
-  (let [operators (fact-operators version)
-        [sql & params] (facts-sql operators query paging-options)]
-    (conj {:results-query (apply vector (sql/paged-sql sql paging-options) params)}
-          (when (:count? paging-options)
-            [:count-query (apply vector (sql/count-sql sql) params)]))))
+  (paging/validate-order-by! (map keyword (keys query/fact-columns)) paging-options)
+  (case version
+    (:v2 :v3)
+    (let [operators (query/fact-operators version)
+          [sql & params] (facts-sql operators query paging-options)]
+      (conj {:results-query (apply vector (jdbc/paged-sql sql paging-options) params)}
+            (when (:count? paging-options)
+              [:count-query (apply vector (jdbc/count-sql sql) params)])))
+
+    (qe/compile-user-query->sql
+     qe/facts-query query paging-options)))
