@@ -2,8 +2,7 @@
   (:import [vimclojure.nailgun NGServer])
   (:require [clojure.string :as string]
             [clojure.tools.nrepl.server :as nrepl]
-            [clojure.tools.nrepl.transport :as nrepl-transport]
-            [swank.swank :as swank]))
+            [clojure.tools.nrepl.transport :as nrepl-transport]))
 
 (defmulti start-repl
   "Starts and instance of the specified `kind` of REPL, listening on `host` and
@@ -19,10 +18,6 @@
   [kind host port]
   (nrepl/start-server :bind host :port port))
 
-(defmethod start-repl "swank"
-  [kind host port]
-  (swank/start-server :host host :port port))
-
 (defmethod start-repl "vimclojure"
   [kind host port]
   (vimclojure.nailgun.NGServer/main (into-array String [(str host ":" port)])))
